@@ -19,19 +19,11 @@ from kraken.util.helpers import NotSet, not_none
 if TYPE_CHECKING:
     from kraken.util.requirements import RequirementSpec
 
-    from kraken.wrapper.lockfile import Lockfile
+    from kraken.wrapper.lockfile import Distribution, Lockfile
 
 logger = logging.getLogger(__name__)
 
 
-@dataclasses.dataclass(frozen=True)
-class Distribution:
-    name: str
-    version: str
-    requirements: list[str]
-    extras: set[str]
-
-
 class BuildEnv(abc.ABC):
     """Interface for the build environment."""
 
@@ -171,7 +163,7 @@ class PexBuildEnv(BuildEnv):
             requirements=requirements.to_args(Path.cwd(), with_options=False),
             index_url=requirements.index_url,
             extra_index_urls=list(requirements.extra_index_urls),
-            transitive=transitive,
+            transitive=True,  # Our lockfiles are not fully cross platform compatible (see kraken-wrapper#2)
         )
 
         layout = {
@@ -250,8 +242,9 @@ class VenvBuildEnv(BuildEnv):
             "--no-python-version-warning",
             "--no-input",
         ]
-        if not transitive:
-            command += ["--no-deps"]
+        # Must enable transitive resolution because lock files are not currently cross platform (see kraken-wrapper#2).
+        # if not transitive:
+        #     command += ["--no-deps"]
         command += requirements.to_args()
         logger.debug("Installing into build environment with Pip: %s", " ".join(command))
         subprocess.check_call(command)
@@ -369,6 +362,8 @@ def _get_environment_for_type(environment_type: BuildEnvType, base_path: Path) -
 
 
 def _get_installed_distributions(kraken_command_prefix: Sequence[str]) -> list[Distribution]:
+    from kraken.wrapper.lockfile import Distribution
+
     command = [*kraken_command_prefix, "query", "env"]
     output = subprocess.check_output(command).decode()
     return [Distribution(x["name"], x["version"], x["requirements"], x["extras"]) for x in json.loads(output)]
