@@ -10,6 +10,7 @@ from pathlib import Path
 from typing import TYPE_CHECKING, NoReturn
 
 from . import __version__
+from .lockfile import calculate_lockfile
 
 if TYPE_CHECKING:
     from kraken.util.requirements import RequirementSpec
@@ -80,8 +81,6 @@ def _get_lock_argument_parser(prog: str) -> argparse.ArgumentParser:
 
 
 def lock(prog: str, argv: list[str], manager: BuildEnvManager, requirements: RequirementSpec) -> NoReturn:
-    from kraken.wrapper.lockfile import Lockfile
-
     parser = _get_lock_argument_parser(prog)
     parser.parse_args(argv)
 
@@ -91,9 +90,13 @@ def lock(prog: str, argv: list[str], manager: BuildEnvManager, requirements: Req
 
     environment = manager.get_environment()
     distributions = environment.get_installed_distributions()
+    lockfile, extra_distributions = calculate_lockfile(requirements, distributions)
+    extra_distributions.discard("pip")  # We'll always have that in a virtual env.
+
+    if extra_distributions:
+        eprint("found extra distributions in build enviroment:", ", ".join(extra_distributions))
 
     had_lockfile = Path(LOCK_FILENAME).exists()
-    lockfile = Lockfile(requirements, {dist.name: dist.version for dist in distributions})
     lockfile.write_to(Path(LOCK_FILENAME))
     manager.set_locked(lockfile)
 
@@ -144,23 +147,29 @@ def _ensure_installed(
     exists = manager.exists()
     install = reinstall or upgrade or not exists
 
+    operation: str
+    reason: str | None = None
+
     if not exists:
         env_type = env_type or env_type or manager.get_environment().get_type()
-        eprint(f"initializing build environment (type: {env_type.name})")
+        operation = "initializing"
     elif upgrade:
-        eprint("upgrading build environment")
+        operation = "upgrading"
     elif reinstall:
-        eprint("reinstalling build environment")
+        operation = "reinstalling"
+    else:
+        operation = "reusing"
 
     current_type = manager.get_environment().get_type()
     if env_type is not None:
         type_changed = exists and env_type != current_type
         if not install and type_changed:
-            eprint(f"re-initializing build environment (type changed: {current_type.name} → {env_type.name})")
             install = True
             manager.remove()
+            operation = "re-initializing"
+            reason = f"type changed from {current_type.name}"
         elif install and type_changed:
-            eprint(f"note: build environment type changed as well ({current_type.name} → {env_type.name})")
+            reason = f"type changed from {current_type.name}"
 
     if not install and exists:
         metadata = manager.get_metadata()
@@ -168,25 +177,37 @@ def _ensure_installed(
             metadata.hash_algorithm
         ):
             install = True
-            eprint("re-initializing build environment (outdated compared to lockfile)")
+            operation = "re-initializing"
+            reason = "outdated compared to lockfile"
         if not lockfile and metadata.requirements_hash != requirements.to_hash(metadata.hash_algorithm):
             install = True
-            eprint("re-initializing build environment (outdated compared to requirements)")
+            operation = "re-initializing"
+            reason = "outdated compared to requirements"
 
     if install:
         if not lockfile or upgrade:
-            logger.info("installing from requirements")
+            source_name = "requirements"
             source = requirements
             transitive = True
         else:
-            logger.info("installing from lock file")
+            source_name = "lock file"
             source = lockfile.to_pinned_requirement_spec()
             lockfile = None
             transitive = False
 
+        env_type = env_type or manager.get_environment().get_type()
+        eprint(
+            operation,
+            "build environment of type",
+            env_type.name,
+            "from",
+            source_name,
+            f"({reason})" if reason else "",
+        )
+
         manager.install(source, env_type, transitive)
     else:
-        eprint(f"reusing build environment (type: {current_type.name})")
+        eprint(operation, "build environment of type", current_type.name)
 
 
 def main() -> NoReturn:
@@ -246,23 +267,26 @@ def main() -> NoReturn:
         manager.remove()
         sys.exit(0)
 
-    _ensure_installed(
-        manager,
-        requirements,
-        lockfile,
-        env_options.reinstall or (os.getenv("KRAKENW_REINSTALL") == "1"),
-        env_options.upgrade,
-        env_options.use,
-    )
-
     cmd: str | None = args.cmd[0] if args.cmd else None
     argv: list[str] = args.cmd[1:] + args.args
 
+    is_lock_command = cmd in ("lock", "l")
+
+    if env_options.any() or not is_lock_command:
+        _ensure_installed(
+            manager,
+            requirements,
+            lockfile,
+            env_options.reinstall or (os.getenv("KRAKENW_REINSTALL") == "1"),
+            env_options.upgrade,
+            env_options.use,
+        )
+
     if cmd is None:
         assert not argv
         sys.exit(0)
 
-    elif cmd in ("lock", "l"):
+    elif is_lock_command:
         lock(f"{parser.prog} lock", argv, manager, requirements)
 
     else:
